// --- [ fdti.c ] -------------------------------------------------------------

#include <windows.h>
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include "ftd2xx.h"
#include <dos.h>
#include <time.h>
#include "xbee_api.h"
#include "global.h"

DWORD rs232_api_write(char* data_out);
void rs232_read(int waiting_for_ACK);
void GetDevice_Status(void);
DWORD GetRx_Bytes(void);
void GetDevice_Info(void);
unsigned int SetDevice_Baudrate(void);
void delay_ms(unsigned int ticks);
unsigned int flush_rx_buffer(void);

// ----------------------------------------------------------------------------
// Schreibt ein vorbereitetes API-Frame via FTDI-USB-Seriell Adapter raus
DWORD rs232_api_write(char* data_out)
{
DWORD data_len;
DWORD data_written, dwErr;

data_len = data_out[2] + (3 + 1);               // + (Start + Lnge) + Prfbyte

ftStatus = FT_Write(ftHandle, data_out, data_len, &data_written);

if (!(ftStatus == FT_OK))
	{
	dwErr = GetLastError();
	printf("Write Failed: %X ", GetLastError());
	return (4);
	}

else
	{
	printf("tx bytes: %X ", data_written);
	printf("Frame_ID: %X \n", last_frame_id);
	return(data_written);
	}
}

// ----------------------------------------------------------------------------
// liest jeweils Byte-Gruppen via serieller Schnittstelle ein.
// wenn mindestens 3 Byte gelesen sind, wird die Lnge der Nachricht aus rs232_in[2]
// bestimmt.
// Sind alle erwarteten Bytes eingetroffen, dann wird beendet.
void rs232_read(int waiting_for_ACK)
{
DWORD dwRead, bytes_read;
unsigned int i, j, last_byte = 0, bytes = 0;
unsigned int wait_ticks;
          
if (waiting_for_ACK == TRUE) wait_ticks = 600;// 600 * 10ms Wartezeit, wenn auf Antwort gewartet wird
else wait_ticks = 100;							// 100 * 10ms Wartezeit, wenn der Empfang von Daten geprft wird

while (GetRx_Bytes() == 0)						// warten, bis die ersten Bytes kommen
	{
	delay_ms(10);
	wait_ticks--;
	if (wait_ticks == 0)                   // Abbrechen, wenn keine Daten eingegangen sind
		{
		if (waiting_for_ACK == TRUE) printf(" -> Keine Antwort\n");
		else printf("no data");
		return;
		}
	}

while(bytes_read = GetRx_Bytes())      	// einlesen, solange wie noch Daten folgen
	{
	ftStatus = FT_Read(ftHandle, &rs232_in[bytes], bytes_read, &dwRead);
	bytes += dwRead;
														// Ein berlauf von rs232_in kann passieren, wenn 
	if (bytes > (sizeof(rs232_in) - 30))	// ein Enddevice den Router mit IO-Daten zuschttet
		{                                   // wenn das nchste Frame lnger als 30 Byte ist,
		printf("\rx-buffer full");          // dann gibt es trotzdem einen berlauf !!!
		break;
		}

	if (last_byte == 0)                 	// wenn die Framelnge noch nicht gelesen wurde
		{
		if (bytes > 2)                   	// wenn mindestens 3 Byte gelesen sind
			{                             	// die Anzahl der erwarteten Bytes bestimmen

			last_byte = rs232_in[2] + (3 + 1);// den Index des letzten Bytes bestimmen
			}
		}
														// beenden, wenn die erwarteten Bytes eingetroffen
	if ((last_byte > 0) && (bytes >= last_byte)) break;
	if (ftStatus != FT_OK) break;       	// beenden, wenn ein Fehler vorliegt.
	delay_ms(100);                    		// eine Pause einlegen, damit weitere Daten empfangen werden knnen
	}

if (rs232_in[0] != FIRST_CHR) printf("\nNo Start Delimiter 0x7E");
printf("\nreceived: %X ", bytes);
if (bytes == last_byte)	printf(" -> Frame complete !\n"); else printf(" -> more than 1 Frame\n");

if (ftStatus == FT_OK)
	{
	if (bytes > 0)
		{
		for (i = 0; i < last_byte; i++) printf("%X ", ((unsigned char) rs232_in[i]));
		if (test_checksum(&rs232_in[0]) > 0) printf(". crc ok\n");else printf(". crc error\n");
		if (bytes > last_byte)
			{
			for (i = last_byte; i < bytes; i++) printf("%X ", rs232_in[i]);
			if (test_checksum(&rs232_in[last_byte]) > 0) printf(". crc ok\n");else printf(". crc error\n");
			}
		}

	else
		{
		printf("no answer\n");
		}
	}

else
	{
	printf("Error reading\n");
	}

//          6                             // die Lngenangabe
//  0   1   2   3   4   5   6   7   8   9 // Byte Nr/Index = Lngenangabe + 3
//              1   2   3   4   5   6     // die Daten
//  1   2   3   4   5   6   7   8   9  10 // die Anzahl = Lngenangabe + 4

// nachfolgende Schleife fr den Fall, dass meherer Frames empfangen werden.
// das ist z.B. der Fall, wenn ein API_Tx_Request() an die eigene Adresse gesendet wird.
j = i = 0;

while (i < bytes)
	{
	if ((rs232_in[j] == FIRST_CHR) && (rs232_in[i+1] == 0) && (i <= bytes))
		{
		i += rs232_in[j+2] + 4;          // Start des nchsten Frames
		if      (rs232_in[j+3] == 0x88) frame88(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x8A) frame8A(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x8B) frame8B(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x90) frame90(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x92) frame92(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x95) frame95(&rs232_in[j]);
		else if (rs232_in[j+3] == 0x97) frame97(&rs232_in[j]);
		else printf("\nFrame Type %X not yet programmed\n\n", rs232_in[j+3]);
		j = i;
		}
	}
}

// ----------------------------------------------------------------------------
// lscht alle empfangenen Bytes aus dem Puffer
unsigned int flush_rx_buffer(void)
{
unsigned int i, bytes = 0;
DWORD dwRead, bytes_read;

while(bytes_read = GetRx_Bytes())      	// einlesen, solange wie noch Daten folgen
	{
	ftStatus = FT_Read(ftHandle, &rs232_in[bytes], bytes_read, &dwRead);
	bytes += dwRead;
	if (ftStatus != FT_OK) break;
	delay_ms(100);
	}

if (bytes > 0)
	{
	printf("\%d Bytes in Buffer : ", bytes);
	for (i = 0; i < bytes; i++) printf("%X ", rs232_in[i]);
	printf("\n");
	}

return(bytes);
}

// ----------------------------------------------------------------------------
// Fragt den RX- und TX-Buffer ab
void GetDevice_Status(void)
{
DWORD rx_queue, tx_queue, event;
ftStatus = FT_GetStatus(ftHandle, &rx_queue, &tx_queue, &event);

if (ftStatus == FT_OK)
	{
	printf("Status: RX: %d TX: %d Event: %d " , rx_queue, tx_queue, event);
	}
else
	{
	printf(" Status: Error ");
	}
}

// ----------------------------------------------------------------------------
// Fragt den Status ab und
// liefert die Anzahl der anstehenden Bytes zurck
DWORD GetRx_Bytes(void)
{
DWORD rx_queue, tx_queue, event;
ftStatus = FT_GetStatus(ftHandle, &rx_queue, &tx_queue, &event);

if (ftStatus == FT_OK) return(rx_queue); else return(0);
}

// ----------------------------------------------------------------------------
// Holt die Device-Bezeichnung
// Public sind: ftHandle, ftStatus
void GetDevice_Info(void)
{
FT_DEVICE ftDevice;
DWORD deviceID;
char SerialNumber[16];
char description[64];

ftStatus = FT_GetDeviceInfo(ftHandle, &ftDevice, &deviceID, SerialNumber, description, NULL);
if (ftStatus == FT_OK)
	{
	printf("\nUSB-Device : ");

	if      (ftDevice == 0) printf("FTU232BM");
	else if (ftDevice == 1) printf("FT232AM");
	else if (ftDevice == 2) printf("FT8100AX");
	else if (ftDevice == 3) printf("FT2232C/L/D");
	else if (ftDevice == 5) printf("FT232R");
	else if (ftDevice == 6) printf("FT2232H");
	else if (ftDevice == 7) printf("FT4232H");
   else printf("unbekannt");

	printf("\nDescription: %s\n", description);
	}

else
	{
	printf("\nDevice-Info failed\n");
	}
}

// ----------------------------------------------------------------------------
// Setzt die bertragungsparameter auf 9600 Baud, 1 Stop-Bit, keine Paritt
unsigned int SetDevice_Baudrate(void)
{
unsigned int success = TRUE;

	ftStatus = FT_SetBaudRate(ftHandle, 9600);
	if (!(ftStatus == FT_OK))
		{
		printf("SetBaudrate failed");
		success = FALSE;
		}
	ftStatus = FT_SetDataCharacteristics(ftHandle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE);
	if (!(ftStatus == FT_OK))
		{
		printf("\nCharacteristics failed\n");
		success = FALSE;
		}

	ftStatus = FT_SetTimeouts(ftHandle, 100, 100);
	if (ftStatus != FT_OK) printf("\nTimeout failed\n");

	if (success) printf("\n9600 Baud, 1 Stop-Bit, no Parity\n");
	return(success);
}

// ----------------------------------------------------------------------------
// fgt an eine Zeichenfolge ein Return an
void add_return(char* txt)
{
unsigned int i = 0;

while (txt[i]) i++;                       // das Textende suchen
txt[i] = 13;                              // Return anhngen
txt[++i] = 0;                             // terminierende 0 anhngen
}

// ----------------------------------------------------------------------------
// wartet im 1 Millisekunden-Takt
void delay_ms(unsigned int ticks)
{
	clock_t time_start, time_end;

	time_start = clock();
	do {
		time_end = clock();
		}
		while ( (time_end - time_start) < ticks);
}

// --- [ eof ] ----------------------------------------------------------------
